      PROGRAM MAIN

C--------------------------------------------------------
C
C  PROGRAMME D'INTEGRATION DES EQUATIONS DE ST VENANT
C  SUR UNE SPHERE. SOLUTIONS EN MOYENNE ZONALE.
C
C  REMARQUES: * U et V sont toujours les vitesses
C               covariantes: u cos(theta), v cos(theta)
C               theta est la latitude
C             * MU=sin(theta)
C
C--------------------------------------------------------

C Toutes les variables utilisees doivent etre definies:

      IMPLICIT NONE
      
C N: nombre de points de grille (points H)
C NITT: Nombre d'iterations temporelles

      INTEGER N,I,NITT,IT
      PARAMETER(N=501)

C MUH, MUU: SIN(THETA) AU POINTS H, HU et HV
C THU, THM: THETA AU POINTS U
C THUM, THUP: BORNES POUR DEFINIR LES FORCAGES.

      DOUBLE PRECISION MUH(N)  ,THH(0:N),THHP,THHM
      DOUBLE PRECISION MUU(0:N),THU(0:N),THUP,THUM

C UB, VB, HB, HUB, HVB: H U V HU ET HV a t-dt (before)

      DOUBLE PRECISION UB(N),VB(N),HB(N),HUB(0:N),HVB(0:N)

C UN, VN, HN, HUN, HVN: H U V HU ET HV a t    (now)

      DOUBLE PRECISION UN(N),VN(N),HN(N),HUN(0:N),HVN(0:N)

C UA, VA, HA, HUA, HVA: H U V HU ET HV a t+dt  (after)

      DOUBLE PRECISION UA(N),VA(N),HA(N),HUA(0:N),HVA(0:N)

C DUDT(N),DVDT(N),DHDT(N): Tendences sur HU, HV ET H

      DOUBLE PRECISION DUDT(N),DVDT(N),DHDT(N)

      DOUBLE PRECISION F0,TIMF,FORC(0:N),FN,FB,TIME
      DOUBLE PRECISION HEAT(N),HRAT

C XPI=PI, RG=G (Cte de gravite), OME: taux de rotation terrestre
C REA: Rayon de la terre, H0: Hauteur du fluide au repos
C RDA: CONSTANTE DE RELAXATION

      DOUBLE PRECISION XPI,RG,OME,H0,RDA,REA,DT
      COMMON/DANGER/XPI,RG,OME,H0,RDA,REA,DT

C  PARAMETRES DU FILTRE DE ASSELIN

      DOUBLE PRECISION ALPHA
      INTEGER N1JOUR,N1HOUR

c  FICHIERS DE STOCKAGE DES RESULTATS
    
      OPEN(10,file='../xmgr/H_E02.dat',form='formatted')
      OPEN(11,file='../xmgr/U_E02.dat',form='formatted')
      OPEN(12,file='../xmgr/V_E02.dat',form='formatted')
      OPEN(13,file='../xmgr/F_E02.dat',form='formatted')

C  INITIALISATIONS DES CONSTANTES:

      XPI=ACOS(-1.)
      REA=6400000.
      OME=2.*XPI/24./3600.
      RG=9.81
      H0=10000.
      RG=6.8
      H0=8000.
      DT=30.
      RDA=5./15./24./3600.
      RDA=0./15./24./3600.
      ALPHA=0.1

      F0=100./24./3600.
      HRAT=100./24./3600.
      FN=0.
      FB=0.

      TIMF=5.*24*3600.
      
      NITT=30.*24.*3600./DT+2
      N1JOUR=24.*3600./DT
      N1HOUR=3600./DT
      


      PRINT *,NITT,N1JOUR,N1HOUR

C  DEFINITION DE LA GRILLE:

      DO I=1,N
         MUH(I)=-1.+2.*FLOAT(I-1)/FLOAT(N-1)    
         THH(I)=ASIN(MUH(I))*180./XPI
      ENDDO
      MUU(0)=-1.
      DO I=1,N-1
         MUU(I)=0.5*(MUH(I)+MUH(I+1))
      ENDDO
      MUU(N)=1.
      DO I=0,N
      THU(I)=ASIN(MUU(I))*180./XPI
      ENDDO

C  INITIALISATIONS (MISE A ZERO) DES CHAMPS PHYSIQUES

      DO I=1,N
         HB(I)=H0
         HN(I)=H0
      ENDDO
      DO I=0,N
         HUB(I)=0.
         HVB(I)=0.
         HUN(I)=0.
         HVN(I)=0.
      ENDDO

C PROFILS DES FORCAGES:

      THHM=-7.5 
      THHP= 7.5 
      DO I=1,N
         HEAT(I)=(0.5-MUH(I)**2)*HRAT
      ENDDO
      THUM=-7.5  
      THUP= 7.5  
      DO I=0,N
         FORC(I)=0.
         IF(THU(I).ge.THUM.and.THU(I).LE.THUP)THEN
         FORC(I)=F0*(THU(I)-THUM)*(THUP-THU(I))/(THUP-THUM)**2*4.
         ENDIF
      ENDDO

C  BOUCLE SUR LE TEMPS

C  FORWARD EULER STEP

      CALL DXDT(N,HUB,HVB,HB,HUB,HVB,MUH,MUU,DUDT,DVDT,DHDT)

      DO I=1,N
         HN(I)=HB(I)+1.*DT*DHDT(I)
      ENDDO
         HUN(0)=0.
         HVN(0)=0.
      DO I=1,N-1
         HUN(I)=HUB(I)+1.*DT*DUDT(I)
         HVN(I)=HVB(I)+1.*DT*DVDT(I)
      ENDDO
         HUN(N)=0.
         HVN(N)=0.

      DO 1 IT=2,NITT


         TIME=(IT-2)*DT


      CALL DXDT(N,HUN,HVN,HN,HUB,HVB,MUH,MUU,DUDT,DVDT,DHDT)

      DO I=1,N
         HA(I)=HB(I)+2.*DT*DHDT(I)
      ENDDO
         HUA(0)=0.
         HVA(0)=0.
      DO I=1,N-1
         HUA(I)=HUB(I)+2.*DT*DUDT(I)
         HVA(I)=HVB(I)+2.*DT*DVDT(I)
      ENDDO
         HUA(N)=0.
         HVA(N)=0.
         

C  FORCAGES

         IF(TIME.GT.0.AND.TIME.LE.TIMF)THEN
         FN=(1.-COS(XPI*TIME/2./TIMF)**2)
         ELSE
         FN=1.
         ENDIF

         DO I=1,N
            HA(I)=HA(I)+2.*DT*(FN*HEAT(I)-RDA*(HN(I)-H0))
         ENDDO
         DO I=1,N-1
            HUA(I)=HUA(I)+DT*HUN(I)/(HN(I+1)+HN(I))*(
     C          FN*(HEAT(I)+HEAT(I+1))-RDA*(HN(I)+HN(I+1)-2.*H0))/2.
            HVA(I)=HVA(I)+DT*HVN(I)/(HN(I+1)+HN(I))*(
     C          FN*(HEAT(I)+HEAT(I+1))-RDA*(HN(I)+HN(I+1)-2.*H0))/2.
         ENDDO

C   FILTRE DE ASSELIN:

      DO I=1,N
         HB(I)=HN(I)+ALPHA/2.*(HA(I)-2.*HN(I)+HB(I))
         HN(I)=HA(I)
      ENDDO
      DO I=0,N
         HUB(I)=HUN(I)+ALPHA/2.*(UA(I)-2.*UN(I)+UB(I))
         HVB(I)=HVN(I)+ALPHA/2.*(VA(I)-2.*VN(I)+VB(I))
         HUN(I)=HUA(I)
         HVN(I)=HVA(I)
      ENDDO

C   STOCKAGE DU RESULTAT TOUT LES JOURS

      if(mod(it,n1jour).eq.1)then
      DO i=1,N
      WRITE(10,*)180./xpi*ASIN(MUH(I)),HB(I)-H0          
      ENDDO
      DO i=1,N-1
      WRITE(11,*)180./xpi*ASIN(MUU(I))
     C           ,(HUB(I))/(HB(I+1)+HB(I))*2.
     C           /SQRT(1-MUU(I)**2)
      WRITE(12,*)180./xpi*ASIN(MUU(I))
     C           ,(HVB(I))/(HB(I+1)+HB(I))*2.
     C           /SQRT(1-MUU(I)**2)
      WRITE(13,*)180./xpi*ASIN(MUU(I))
     C                ,FN*HEAT(I)
      ENDDO
      WRITE(10,*)' '
      WRITE(11,*)' '
      WRITE(12,*)' '
      WRITE(13,*)' '
      endif

1     CONTINUE !  FIN DE LA BOUCLE SUR LE TEMPS

      STOP
      END
